/*
	Autor        : Frank Link
	WEB          : www.derplatinenshop.de
	Datum        : 22.05.2008
	Ursprung     : http://www.roboternetz.de/wissen/index.php/Portexpander_am_AVR#Makros.2C_Datenstrukturen.2C_Funktionen
	Beschreibung : Bibliothek zur Ausgabe und Ansteuerung von 74HC595 ber einen ATMEL Mikrokontroller.
				   Die Ansteuerung erfolgt ber Software SPI
	CPU			 : ATMEGA 8
*/

#include <avr/io.h>
#include <stdlib.h>
#include "Com74hc595.h"

// Array fuer die Daten
unsigned char com74hc595[ COM74HC595_SIZE ];

/*
	Autor        : Frank Link
	Datum        : 22.05.2008
	Beschreibung : ermittelt an Hand der BitNumber den Port und 
	               das im Port zusetzende Bit, setzt das Bit auf logisch 1
*/
void Com74hc595_setBit( unsigned char BitNumber)
{
	div_t value = div( BitNumber, COM74HC595_BYTES );
	com74hc595[ value.quot ] |= (1 << ( value.rem ) );
}

/*
	Autor        : Frank Link
	Datum        : 22.05.2008
	Beschreibung : ermittelt an Hand der BitNumber den Port und 
	               das im Port zusetzende Bit, setzt das Bit auf logisch 0
*/
void Com74hc595_unsetBit( unsigned char BitNumber)
{
	div_t value = div( BitNumber, COM74HC595_BYTES );
	com74hc595[ value.quot ] &= ~(1 << ( value.rem ) );
}

/*
	Autor        : Frank Link
	Datum        : 22.05.2008
	Beschreibung : setzt alle Bits im Array auf logisch 1
*/
void Com74hc595_setall()
{
	
	for ( int i = 0; i < COM74HC595_BYTES; i++ )
	{
		Com74hc595_setBit( i );
	}
}

/*
	Autor        : Frank Link
	Datum        : 22.05.2008
	Beschreibung : setzt alle Bits im Array auf logisch 0
*/
void Com74hc595_unsetall()
{
	for ( int i = 0; i < COM74HC595_BYTES; i++ )
	{
		Com74hc595_unsetBit( i );
	}
}

/*
	Autor        : Frank Link
	Datum        : 22.05.2008
	Beschreibung : setzt eine konkreten Port auf das mit Bits
	               bergebene Muster
*/
void Com74hc595_setPort( unsigned char Port, unsigned char Bits )
{
	com74hc595[ Port ] = Bits;
}

/*
	Autor        : Frank Link
	Datum        : 22.05.2008
	Beschreibung : setzt fr einen konkreten Port alle Bits
	               auf logisch 0
*/
void Com74hc595_unsetPort( unsigned char Port )
{
	com74hc595[ Port ] = 0x00;
}


/*
	Autor        : Frank Link
	Datum        : 22.05.2008
	Beschreibung : Initialisierung der Ports
*/
void Com74hc595_init()
{
    /* SER auf definierten Level LOW */
	PORT_COM74HC595 &= ~(1<<PORT_SER);

    /* SCR und RCK auf definierten Level HIGH */
	PORT_COM74HC595 |= (1<<PORT_SCK);
	PORT_COM74HC595 |= (1<<PORT_RCK);

    /* Verwendete Ports auf OUT */
	DDR_COM74HC595 |= ( 1 << DDR_SER);
	DDR_COM74HC595 |= ( 1 << DDR_SCK);
	DDR_COM74HC595 |= ( 1 << DDR_RCK);

	Com74hc595_unsetall();
	Com74hc595_out();     
}

/*
	Autor        : Frank Link
	Datum        : 22.05.2008
	Beschreibung : Arraymuster an die Ports ausgeben
*/
void Com74hc595_out()
{
    unsigned char anz = COM74HC595_SIZE;
    unsigned char* serp = com74hc595 + COM74HC595_SIZE;

    do
    {
        unsigned char bits;
        unsigned char data = *--serp;

        /* 8 Bits pro Byte rausschieben */
        for (bits = 8; bits > 0; bits--)
        {
			PORT_COM74HC595 &= ~(1<<PORT_SER);
            if (data & 0x80)
            {
				PORT_COM74HC595 |= (1<<PORT_SER);
            };

            data <<= 1;
            /* Strobe an SCK schiebt Daten im Gaensemarsch */
            /* um 1 Position weiter durch alle Schieberegister */
			PORT_COM74HC595 &= ~(1<<PORT_SCK);
			PORT_COM74HC595 |= (1<<PORT_SCK);
        }
    }
    while (--anz > 0);

    /* Strobe an RCK bringt die Daten von den Schieberegistern in die Latches */
	PORT_COM74HC595 &= ~(1<<PORT_RCK);
	PORT_COM74HC595 |= (1<<PORT_RCK);
}
