#include <avr/io.h>
#include "com74hc595.h"
#include "com74hc595_lcd.h"
#include <util/delay.h>
 
// sendet ein Datenbyte an das LCD
 
void com74hc595_lcd_data(unsigned char temp1)
{
 	unsigned char temp2 = temp1;
	unsigned char value = 0x00;
 
   	temp1 = temp1 >> 4;
   	temp1 = temp1 & 0x0F;

	value = 0x00;
	value |= ( 1 << LCD_RS );		// RS = 1
	value |= temp1;
	Com74hc595_setPort( LCD_PORT, value );
	Com74hc595_out();
   	com74hc595_lcd_enable();
 
   	temp2 = temp2 & 0x0F;
	value = 0x00;
	value |= ( 1 << LCD_RS );		// RS = 1
	value |= temp2;
	Com74hc595_setPort( LCD_PORT, value );
	Com74hc595_out();
   	com74hc595_lcd_enable();
 
   	_delay_us(420);
}
 
// sendet einen Befehl an das LCD
 
void com74hc595_lcd_command(unsigned char temp1)
{
   	unsigned char temp2 = temp1;
	unsigned char value = 0x00;

	_delay_ms( 10 ); 

   	temp1 = temp1 >> 4;              // oberes Nibble holen
   	temp1 = temp1 & 0x0F;            // maskieren

   	value = 0x00;					 // RS = 0
   	value |= temp1;

	Com74hc595_setPort( LCD_PORT, value );
	Com74hc595_out();
   	com74hc595_lcd_enable();

   	temp2 = temp2 & 0x0F;            // unteres Nibble holen und maskieren
   	value = 0x00;					 // RS = 0
   	value |= temp2;

	Com74hc595_setPort( LCD_PORT, value );
	Com74hc595_out();
   	com74hc595_lcd_enable();

  	_delay_us(10);
}
 
// erzeugt den Enable-Puls
void com74hc595_lcd_enable(void)
{
	Com74hc595_setBit( LCD_EN );
	Com74hc595_out();
    _delay_us(10);                   // kurze Pause
	Com74hc595_unsetBit( LCD_EN );
	Com74hc595_out();
}
 
void com74hc595_lcd_init(void)
{
   	// muss 3mal hintereinander gesendet werden zur Initialisierung
   	_delay_ms(250);

   	com74hc595_lcd_command(0x30);
   	_delay_ms(5);
	com74hc595_lcd_enable();
   	_delay_ms(1);
	com74hc595_lcd_enable();
   	_delay_ms(1);
 
   	// 4Bit / 2 Zeilen / 5x7
   	com74hc595_lcd_command( 0x28 );
  	// Display ein / Cursor aus / kein Blinken
   	com74hc595_lcd_command( 0x0C ); 
   	// inkrement / kein Scrollen
   	com74hc595_lcd_command( 0x06 );
	// Display lschen
   	com74hc595_lcd_clear();
	com74hc595_lcd_home();
}
 
// Sendet den Befehl zur Lschung des Displays
 
void com74hc595_lcd_clear(void)
{
   com74hc595_lcd_command( CLEAR_DISPLAY );
//   _delay_ms(5);
}
 
// Sendet den Befehl: Cursor Home
 
void com74hc595_lcd_home(void)
{
   com74hc595_lcd_command( CURSOR_HOME );
   _delay_ms(5);
}
 
// setzt den Cursor in Zeile y (1..4) Spalte x (0..15)
 
void com74hc595_set_cursor(uint8_t x, uint8_t y)
{
  uint8_t tmp = 0x00;
 
  switch (y) {
    case 1: tmp=0x80+0x00+x; break;    // 1. Zeile
    case 2: tmp=0x80+0x40+x; break;    // 2. Zeile
    case 3: tmp=0x80+0x10+x; break;    // 3. Zeile
    case 4: tmp=0x80+0x50+x; break;    // 4. Zeile
  }
  com74hc595_lcd_command(tmp);
}
 
// Schreibt einen String auf das LCD
 
void com74hc595_lcd_string(char *data)
{
    while(*data) 
	{
        com74hc595_lcd_data(*data);
        data++;
    }
}

